# Generated by Django 4.2.4 on 2023-08-25 10:29

import budgetmanager.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('budgetmanager', '0002_budget_last_modified_budget_modified_by_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='ShareCode',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('can_edit', models.BooleanField(default=False)),
                ('expiry', models.DateTimeField(default=budgetmanager.models._get_sharecode_expiry, editable=False)),
                ('added_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('budget', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='budgetmanager.budget')),
            ],
        ),
    ]
